------------------------------------
--) Duplicate Selection in Pattern
------------------------------------

renoise.tool():add_keybinding {
  name = "Global:Tools: Duplicate Selection In Pattern", 
  invoke = function()duplicate_selection() end  
}

--DUPLICATE SELECTION IN PATTERN
--------------------------------
function duplicate_selection()
  --song
  local song = renoise.song()
  local pattern_index = song.selected_pattern_index
  --get current selection
  local selection = song.selection_in_pattern
  
  --return if no selection in pattern editor
  if selection == nil then
    return
  end
  
  --selection ranges
  local start_track = selection.start_track
  local end_track = selection.end_track
  local start_line = selection.start_line
  local end_line = selection.end_line
  local start_column = selection.start_column
  local end_column = selection.end_column
  
  if end_line == #song.selected_pattern_track.lines then
   return
  end
  
  --if selection is within one track we need set the end column accordingly in the loop
  --else it is left at 12 (default)
  local first_track_end_column = 12
  if start_track == end_track then
    first_track_end_column = end_column 
  end
  
  local stored_pattern_lines = {}
  local range_length = end_line - start_line + 1 --minimum range = 1
  local end_of_original_selection = end_line
  
  --if there will be over-run on the end of the pattern then limit the scope by changing the end line
  if (end_line + range_length) > #song.selected_pattern_track.lines then
    end_line =  #song.selected_pattern_track.lines - range_length
   -- range_length = end_line - start_line 
  end
  
  ----------------------
  --loop tracks 
  for i = start_track,end_track do
    --refresh pattern_track for each loop
    local pattern_track = song:pattern(pattern_index):track(i)
    local current_track = song:track(i)
    --first track
    if i == start_track then
      --loop lines in track
      for line = start_line,end_line do
        --set target line
        local target_line = line + range_length
        --NOTE COLUMNS IF PRESENT IN TRACK
        if current_track.type  == renoise.Track.TRACK_TYPE_SEQUENCER then
          for col = start_column, first_track_end_column do
            --copy from source note to target
            pattern_track:line(target_line):note_column(col):copy_from(pattern_track:line(line):note_column(col)) 
          end
        end
        --FX COLUMNS
        for fx_col = 1,8 do
          if pattern_track:line(line):effect_column(fx_col).is_selected then
            pattern_track:line(target_line):effect_column(fx_col):copy_from(pattern_track:line(line):effect_column(fx_col)) 
          end   
        end
      end
   
    --last track
    elseif (i == end_track) and (start_track ~= end_track) then
      --loop lines in track
      for line = start_line,end_line do
      --set target line
        local target_line = line + range_length
        --NOTE COLUMNS IF PRESENT IN TRACK
        if current_track.type == renoise.Track.TRACK_TYPE_SEQUENCER then
          for col = 1, end_column do
            --copy from source note to target
            pattern_track:line(target_line):note_column(col):copy_from(pattern_track:line(line):note_column(col)) 
          end
        end
        --FX COLUMNS
        for fx_col = 1,8 do
          if pattern_track:line(line):effect_column(fx_col).is_selected then
            pattern_track:line(target_line):effect_column(fx_col):copy_from(pattern_track:line(line):effect_column(fx_col)) 
          end   
        end
      end
    
     --all tracks in between (copy whole lines as all columns must be selected here)
     elseif (i ~= start_track) and (i~= end_track) then

       --just loop lines in this track (no col loop or separate fx loop required as all columns are selected)
       for line = start_line,end_line do
        --set target line
         local target_line = line + range_length
         --copy whole line `THIS INCLUDES FX COLUMNS`
         pattern_track:line(target_line):copy_from(pattern_track:line(line))
      end
    end
  end
  
  --update pattern selection to where all the data has been pasted
  start_line = end_of_original_selection + 1 --the next line
  end_line = start_line + range_length - 1
    
  if start_line > #song.selected_pattern_track.lines then
    start_line = #song.selected_pattern_track.lines
  end
  
  if end_line > #song.selected_pattern_track.lines then
    end_line = #song.selected_pattern_track.lines
  end

  --set renoise selection to new range
  song.selection_in_pattern = { start_line = start_line,
                                start_track = start_track,
                                end_line = end_line,
                                end_track = end_track,
                                start_column = start_column,
                                end_column = end_column
                               } 
  --update cursor position
  local pos = renoise.song().transport.edit_pos
  pos.line = start_line 
  renoise.song().transport.edit_pos = pos
  
end




